// Copyright (C) 2024, Jakob Wakeling
// All rights reserved.

#include "../keys.h"

#include <xcb/xcb_keysyms.h>
#include <X11/keysym.h>

struct keymap { xcb_keysym_t keysym; draw_key_k key; };

const struct keymap keymap[] = {
	{ XK_space,      DRAW_KEY_SPACE      },
    { XK_apostrophe, DRAW_KEY_APOSTROPHE },
    { XK_comma,      DRAW_KEY_COMMA      },
    { XK_minus,      DRAW_KEY_MINUS      },
    { XK_period,     DRAW_KEY_PERIOD     },
    { XK_slash,      DRAW_KEY_SLASH      },

	{ XK_0, DRAW_KEY_0 },
	{ XK_1, DRAW_KEY_1 },
	{ XK_2, DRAW_KEY_2 },
	{ XK_3, DRAW_KEY_3 },
	{ XK_4, DRAW_KEY_4 },
	{ XK_5, DRAW_KEY_5 },
	{ XK_6, DRAW_KEY_6 },
	{ XK_7, DRAW_KEY_7 },
	{ XK_8, DRAW_KEY_8 },
	{ XK_9, DRAW_KEY_9 },

	{ XK_semicolon, DRAW_KEY_SEMICOLON },
	{ XK_equal,     DRAW_KEY_EQUAL     },

	{ XK_a, DRAW_KEY_A },
	{ XK_b, DRAW_KEY_B },
	{ XK_c, DRAW_KEY_C },
	{ XK_d, DRAW_KEY_D },
	{ XK_e, DRAW_KEY_E },
	{ XK_f, DRAW_KEY_F },
	{ XK_g, DRAW_KEY_G },
	{ XK_h, DRAW_KEY_H },
	{ XK_i, DRAW_KEY_I },
	{ XK_j, DRAW_KEY_J },
	{ XK_k, DRAW_KEY_K },
	{ XK_l, DRAW_KEY_L },
	{ XK_m, DRAW_KEY_M },
	{ XK_n, DRAW_KEY_N },
	{ XK_o, DRAW_KEY_O },
	{ XK_p, DRAW_KEY_P },
	{ XK_q, DRAW_KEY_Q },
	{ XK_r, DRAW_KEY_R },
	{ XK_s, DRAW_KEY_S },
	{ XK_t, DRAW_KEY_T },
	{ XK_u, DRAW_KEY_U },
	{ XK_v, DRAW_KEY_V },
	{ XK_w, DRAW_KEY_W },
	{ XK_x, DRAW_KEY_X },
	{ XK_y, DRAW_KEY_Y },
	{ XK_z, DRAW_KEY_Z },

	{ XK_bracketleft,  DRAW_KEY_LEFT_BRACKET  },
	{ XK_backslash,    DRAW_KEY_BACKSLASH     },
	{ XK_bracketright, DRAW_KEY_RIGHT_BRACKET },
	{ XK_grave,        DRAW_KEY_TILDE         },
	{ XK_Escape,       DRAW_KEY_ESCAPE	      },
	{ XK_BackSpace,    DRAW_KEY_BACKSPACE     },
	{ XK_Tab,          DRAW_KEY_TAB           },
	{ XK_Caps_Lock,    DRAW_KEY_CAPS_LOCK     },
	{ XK_Return,       DRAW_KEY_ENTER         },

	{ XK_F1,  DRAW_KEY_F1  },
	{ XK_F2,  DRAW_KEY_F2  },
	{ XK_F3,  DRAW_KEY_F3  },
	{ XK_F4,  DRAW_KEY_F4  },
	{ XK_F5,  DRAW_KEY_F5  },
	{ XK_F6,  DRAW_KEY_F6  },
	{ XK_F7,  DRAW_KEY_F7  },
	{ XK_F8,  DRAW_KEY_F8  },
	{ XK_F9,  DRAW_KEY_F9  },
	{ XK_F10, DRAW_KEY_F10 },
	{ XK_F11, DRAW_KEY_F11 },
	{ XK_F12, DRAW_KEY_F12 },
	{ XK_F13, DRAW_KEY_F13 },
	{ XK_F14, DRAW_KEY_F14 },
	{ XK_F15, DRAW_KEY_F15 },
	{ XK_F16, DRAW_KEY_F16 },
	{ XK_F17, DRAW_KEY_F17 },
	{ XK_F18, DRAW_KEY_F18 },
	{ XK_F19, DRAW_KEY_F19 },
	{ XK_F20, DRAW_KEY_F20 },
	{ XK_F21, DRAW_KEY_F21 },
	{ XK_F22, DRAW_KEY_F22 },
	{ XK_F23, DRAW_KEY_F23 },
	{ XK_F24, DRAW_KEY_F24 },

	{ XK_Sys_Req,     DRAW_KEY_SYSRQ       },
	{ XK_Scroll_Lock, DRAW_KEY_SCROLL_LOCK },
	{ XK_Pause,       DRAW_KEY_PAUSE       },
	{ XK_Insert,      DRAW_KEY_INSERT      },
	{ XK_Delete,      DRAW_KEY_DELETE      },
	{ XK_Home,        DRAW_KEY_HOME        },
	{ XK_End,         DRAW_KEY_END         },
	{ XK_Page_Up,     DRAW_KEY_PAGE_UP     },
	{ XK_Page_Down,   DRAW_KEY_PAGE_DOWN   },
	{ XK_Left,        DRAW_KEY_LEFT        },
	{ XK_Up,          DRAW_KEY_UP          },
	{ XK_Right,       DRAW_KEY_RIGHT       },
	{ XK_Down,        DRAW_KEY_DOWN        },

	{ XK_Num_Lock,     DRAW_KEY_NUM_LOCK    },
	{ XK_KP_Divide,    DRAW_KEY_KP_SLASH    },
	{ XK_KP_Multiply,  DRAW_KEY_KP_ASTERISK },
	{ XK_KP_Subtract,  DRAW_KEY_KP_MINUS    },
	{ XK_KP_Add,       DRAW_KEY_KP_PLUS     },
	{ XK_KP_Enter,     DRAW_KEY_KP_ENTER    },
	{ XK_KP_Insert,    DRAW_KEY_KP_0        },
	{ XK_KP_End,       DRAW_KEY_KP_1        },
	{ XK_KP_Down,      DRAW_KEY_KP_2        },
	{ XK_KP_Page_Down, DRAW_KEY_KP_3        },
	{ XK_KP_Left,      DRAW_KEY_KP_4        },
	{ XK_KP_Begin,     DRAW_KEY_KP_5        },
	{ XK_KP_Right,     DRAW_KEY_KP_6        },
	{ XK_KP_Home,      DRAW_KEY_KP_7        },
	{ XK_KP_Up,        DRAW_KEY_KP_8        },
	{ XK_KP_Page_Up,   DRAW_KEY_KP_9        },
	{ XK_KP_Delete,    DRAW_KEY_KP_DOT      },

	{ XK_Shift_L,   DRAW_KEY_LEFT_SHIFT    },
	{ XK_Control_L, DRAW_KEY_LEFT_CONTROL  },
	{ XK_Alt_L,     DRAW_KEY_LEFT_ALT      },
	{ XK_Super_L,   DRAW_KEY_LEFT_SUPER    },
	{ XK_Shift_R,   DRAW_KEY_RIGHT_SHIFT   },
	{ XK_Control_R, DRAW_KEY_RIGHT_CONTROL },
	{ XK_Alt_R,     DRAW_KEY_RIGHT_ALT     },
	{ XK_Super_R,   DRAW_KEY_RIGHT_SUPER   },
};

const size_t keymap_size = sizeof(keymap) / sizeof(*keymap);
